@@ -5,21 +5,21 @@ Numpy array with the additional attribute `unit`.
 from __future__ import annotations
 
 from typing import Callable, Union
-from numbers import Real
 
 import numpy as np
 from numpy.core._exceptions import UFuncTypeError
 
 from .errors import OperationNotSupported
+from .quantity import Quantity
 from .unit import UnitBase, UnitExprBase
 
 
-class QArray(np.ndarray):
+class qarray(np.ndarray):
     """
     An array with elements representing a quantity that can be
     described by a numerical value and a unit.
 
-    `QArray` is a sub-class of ndarray with the additional instance
+    `qarray` is a sub-class of ndarray with the additional instance
     attribute `unit`.
 
     Implementation closely follows:
@@ -40,7 +40,7 @@ class QArray(np.ndarray):
         unit=1.0,
     ):
         # The call in the next line triggers a call to
-        # QArray.__array_finalize__
+        # qarray.__array_finalize__
         obj = super().__new__(
             subtype, shape, dtype, buffer, offset, strides, order
         )
@@ -49,46 +49,40 @@ class QArray(np.ndarray):
 
     def __array_finalize__(self, obj):
         # ``self`` is a new object resulting from
-        # ndarray.__new__(QArray, ...), therefore it only has
+        # ndarray.__new__(qarray, ...), therefore it only has
         # attributes that the ndarray.__new__ constructor gave it -
         # i.e. those of a standard ndarray.
         #
         # We could have got to the ndarray.__new__ call in 3 ways:
-        # From an explicit constructor - e.g. QArray():
+        # From an explicit constructor - e.g. qarray():
         #    obj is None
-        #    (we're in the middle of the QArray.__new__
+        #    (we're in the middle of the qarray.__new__
         #    constructor, and self.unit will be set when we return to
-        #    QArray.__new__)
+        #    qarray.__new__)
         if obj is None:
             return
-        # From view casting - e.g arr.view(QArray):
+        # From view casting - e.g arr.view(qarray):
         #    obj is arr
-        #    (type(obj) can be QArray)
+        #    (type(obj) can be qarray)
         # From new-from-template - e.g infoarr[:3]
-        #    type(obj) is QArray
+        #    type(obj) is qarray
         #
         # Note that it is here, rather than in the __new__ method,
         # that we set the default value for 'unit', because this
         # method sees all creation of default objects - with the
-        # QArray.__new__ constructor, but also with
-        # arr.view(QArray).
+        # qarray.__new__ constructor, but also with
+        # arr.view(qarray).
         self.__unit = getattr(obj, "unit", 1.0)
 
     @classmethod
-    def from_input(cls, input, unit=1.0) -> QArray:
-        """Constructs a `QArray` from an existing ndarray
+    def from_input(cls, input, unit=1.0) -> qarray:
+        """Constructs a `qarray` from an existing ndarray
         or from a (nested) sequence of entries.
         """
         obj = np.asarray(input).view(cls)
         obj.unit = unit
         return obj
 
-    def __str__(self) -> str:
-        return super().__str__() + " unit: " + str(self.unit)
-
-    def __repr__(self) -> str:
-        return super().__repr__()[:-1] + f", unit={self.unit})"
-
     @property
     def unit(self):
         """Returns the unit of the object."""
@@ -96,7 +90,11 @@ class QArray(np.ndarray):
 
     @unit.setter
     def unit(self, value) -> None:
-        factor = value.factor if isinstance(value, self._unit_types) else value
+        factor = (
+            value.factor
+            if isinstance(value, self._unit_types)
+            else float(value)
+        )
 
         if factor == 1.0:
             self.__unit = value
@@ -120,7 +118,13 @@ class QArray(np.ndarray):
                 # normalize unit:
                 self.__unit = value
 
-    def __add__(self, other) -> QArray:
+    def __str__(self) -> str:
+        return super().__str__() + " unit: " + str(self.unit)
+
+    def __repr__(self) -> str:
+        return super().__repr__()[:-1] + f", unit={self.unit})"
+
+    def __add__(self, other) -> qarray:
         other_unit = getattr(other, "unit", 1.0)
         # If units match simply add arrays.
         if self.unit == other_unit:
@@ -128,7 +132,7 @@ class QArray(np.ndarray):
 
         alpha = other_unit / self.unit
 
-        if isinstance(alpha, Real):
+        if isinstance(alpha, float):
             return super().__add__(other * alpha)
 
         if (
@@ -139,7 +143,7 @@ class QArray(np.ndarray):
 
         raise OperationNotSupported(self, other, "+")
 
-    def __sub__(self, other) -> QArray:
+    def __sub__(self, other) -> qarray:
         other_unit = getattr(other, "unit", 1.0)
 
         # If units match simply subtract arrays.
@@ -148,7 +152,7 @@ class QArray(np.ndarray):
 
         alpha = other_unit / self.unit
 
-        if isinstance(alpha, Real):
+        if isinstance(alpha, float):
             return super().__sub__(other * alpha)
 
         if (
@@ -159,7 +163,7 @@ class QArray(np.ndarray):
 
         raise OperationNotSupported(self, other, "-")
 
-    def __mul__(self, other) -> QArray:
+    def __mul__(self, other) -> qarray:
         """
         Returns the result of multiplying the united ndarray `self`
         with `other`.
@@ -169,12 +173,17 @@ class QArray(np.ndarray):
             obj.unit = self.unit * other
             return obj
 
+        if isinstance(other, Quantity):
+            obj = super().__mul__(other.value)
+            obj.unit = self.unit * other.unit
+            return obj
+
         obj = super().__mul__(other)
         other_unit = getattr(other, "unit", 1.0)
         obj.unit = self.unit * other_unit
         return obj
 
-    def __rmul__(self, other) -> QArray:
+    def __rmul__(self, other) -> qarray:
         """
         Returns the result of multiplying `other` with the united array
         `self`.
@@ -184,12 +193,17 @@ class QArray(np.ndarray):
             obj.unit = other * self.unit
             return obj
 
+        if isinstance(other, Quantity):
+            obj = super().__rmul__(other.value)
+            obj.unit = other.unit * self.unit
+            return obj
+
         obj = super().__rmul__(other)
         other_unit = getattr(other, "unit", 1.0)
         obj.unit = other_unit * self.unit
         return obj
 
-    def __truediv__(self, other) -> QArray:
+    def __truediv__(self, other) -> qarray:
         """
         Returns the result of dividing the united ndarray `self`
         with `other`.
@@ -199,69 +213,91 @@ class QArray(np.ndarray):
             obj.unit = self.unit / other
             return obj
 
+        if isinstance(other, Quantity):
+            obj = super().__truediv__(other.value)
+            obj.unit = self.unit / other.unit
+            return obj
+
         obj = super().__truediv__(other)
         other_unit = getattr(other, "unit", 1.0)
         obj.unit = self.unit / other_unit
         return obj
 
-    def __pow__(self, other: Union[float, int]) -> QArray:
+    def __rtruediv__(self, other) -> qarray:
+        """
+        Returns the result of dividing `other` by the united ndarray `self`.
+        """
+        if isinstance(other, self._unit_types):
+            obj = super().__rtruediv__(other.factor)
+            obj.unit = other / self.unit
+            return obj
+
+        if isinstance(other, Quantity):
+            obj = super().__rtruediv__(other.value)
+            obj.unit = other.unit / self.unit
+            return obj
+
+        obj = super().__rtruediv__(other)
+        other_unit = getattr(other, "unit", 1.0)
+        obj.unit = other_unit / self.unit
+        return obj
+
+    def __pow__(self, other: Union[float, int]) -> qarray:
         obj = super().__pow__(other)
         obj.unit = self.unit.__pow__(other)
         return obj
 
-    def __abs__(self) -> QArray:
+    def __abs__(self) -> qarray:
         obj = super().__abs__()
         obj.unit = self.unit
         return obj
 
-    def __neg__(self) -> QArray:
+    def __neg__(self) -> qarray:
         obj = super().__neg__()
         obj.unit = self.unit
         return obj
 
-    def __pos__(self) -> QArray:
+    def __pos__(self) -> qarray:
         obj = super().__pos__()
         obj.unit = self.unit.__pos__()
         return obj
 
-    def __eq__(self, other) -> QArray:
-        result = self.__compare(other, super().__eq__)
+    def __eq__(self, other) -> qarray:
+        result = self.compare(other, super().__eq__)
         if result is None:
             result = super().__eq__(other)
-            result.__unit = 1.0
+            result.unit = 1.0
             result.fill(False)
         return result
 
-    def __le__(self, other) -> QArray:
-        result = self.__compare(other, super().__le__)
+    def __le__(self, other) -> qarray:
+        result = self.compare(other, super().__le__)
         if result is None:
             raise OperationNotSupported(self, other, "<=")
         return result
 
-    def __ge__(self, other) -> QArray:
-        result = self.__compare(other, super().__ge__)
+    def __ge__(self, other) -> qarray:
+        result = self.compare(other, super().__ge__)
         if result is None:
             raise OperationNotSupported(self, other, ">=")
         return result
 
-    def __gt__(self, other) -> QArray:
-        result = self.__compare(other, super().__gt__)
+    def __gt__(self, other) -> qarray:
+        result = self.compare(other, super().__gt__)
         if result is None:
             raise OperationNotSupported(self, other, ">")
         return result
 
-    def __lt__(self, other) -> QArray:
-        result = self.__compare(other, super().__lt__)
+    def __lt__(self, other) -> qarray:
+        result = self.compare(other, super().__lt__)
         if result is None:
             raise OperationNotSupported(self, other, "<")
         return result
 
-    def __compare(
-        self, other, comparison_operator: Callable
-    ) -> Union[QArray, None]:
+    def compare(self, other, comparison_operator: Callable) -> qarray:
         """
         Generic comparison function that handles input of type `Number`,
-        `ndarray` and `QArray` using `comparison_operator`.
+        `ndarray` and `qarray` using `comparison_operator`.
 
         Returns `None` if the comparison failed due to incompatible units.
         """
@@ -269,14 +305,14 @@ class QArray(np.ndarray):
 
         if self.unit == other_unit:
             obj = comparison_operator(other)
-            obj.__unit = 1.0
+            obj.unit = 1.0
             return obj
 
         alpha = other_unit / self.unit
 
-        if isinstance(alpha, Real):
+        if isinstance(alpha, float):
             obj = comparison_operator(other * alpha)
-            obj.__unit = 1.0
+            obj.unit = 1.0
             return obj
 
         if (
@@ -284,7 +320,7 @@ class QArray(np.ndarray):
             and alpha.base_exponents == alpha.base_exponents_zero
         ):
             obj = comparison_operator(other * alpha.base_factor)
-            obj.__unit = 1.0
+            obj.unit = 1.0
             return obj
 
         return None
